/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is Mozilla JSIRC Library.
 *
 * The Initial Developer of the Original Code is
 * New Dimensions Consulting, Inc.
 * Portions created by the Initial Developer are Copyright (C) 1999
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *   Robert Ginda, rginda@ndcico.com, original author
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

body {

    margin: 0px 0px 0px 0px;
    background: black;

}

.bold {

    font-weight: bold;

}

.italic {

    font-style: italic;

}

.underline {

    text-decoration: underline;

}

.strikethrough {

    text-decoration: line-through;

}

.teletype {

    font-family: monospace;

}

.smallcap {

    font-variant: small-caps;

}

.rheet {

    font-size: 14pt;
    font-weight: bold;
    color: magenta;

}

/* output from a chat session (contains msgs) */
.chat-view {

	vertical-align: text-bottom;

}

/* common container for all portions of a message
 * (contains msg-*s) */
.msg {

    font-size: 10pt;
	font-family: sans-serif;

}

.msg[user="!ME"] {

	background: lightgrey;

}

/* message data in output window */
.msg-data {

    font-weight: bold;
    color: lightgrey;
    background: #1a2a44;

}

/* message data in output window */

.msg-data[user="!ME"]{

    background: black;

}

.msg-data[msgtype="JOIN"],
.msg-data[msgtype="PART"] {

    width: 100%;
    font-variant: small-caps;
    background: lightgray;
    color: black;

}

.msg-data[msgtype="HELLO"] {

    background: white;
	color: darkgreen;

}

.msg-data[msgtype="ERROR"] {

    background: red;
	color: white;

}

.msg-data[msgtype="USAGE"] {

    font-style: italic;
	color: white;

}

.msg-data[msgtype="HELP"] {

    font-weight: normal;

}

.msg-data[msgtype="ACTION"] {

	color: cyan;

}

.msg-data[msgtype="NOTICE"] {

	color: yellow;

}

.msg-data[msgtype="KICK"] {

    background: orange;
	color: yellow;

}

.msg-data[msgtype="QUIT"] {

    background: lightgrey;
	color: brown;

}

/* nickname field in output */
.msg-user {
	
    text-align: center;
    vertical-align: middle;
    color: lightgrey;
	font-weight: bold;
    background: grey;

}

.msg-user[parity="odd"]{

    background: black;

}

.msg-user[user="!ME"] {

    color : white;

}

.msg-user[msgtype="ACTION"] {

	font-style: italic;

}

/* Message type indicator in output window */
.msg-type {

    text-align: center;
    vertical-align: middle;
	color: brown;
	font-weight: bold;
    background: lightgrey;

}

.msg-type[user="!ME"] {

    background: silver;

}
